@@ -70,16 +70,9 @@ PyDUChainTest::PyDUChainTest(QObject* parent): QObject(parent)
         qFatal("Failed find data directory for test files. Aborting");
     }
 
-    char tempdirname[] = "/tmp/kdev-python-test.XXXXXX";
-    if (!mkdtemp(tempdirname)) {
-        qFatal("Failed to create temp directory, Aboring");
-    }
-    testDir = QDir(QString(tempdirname));
-    qCDebug(KDEV_PYTHON_DUCHAIN) << "tempdirname" << tempdirname;
+    testDir = QDir(testDirOwner.path());
 
-    QByteArray pythonpath = qgetenv("PYTHONPATH");
-    pythonpath.prepend(":").prepend(assetsDir.absolutePath().toUtf8());
-    qputenv("PYTHONPATH", pythonpath);
+    qputenv("PYTHONPATH", assetsDir.absolutePath().toUtf8());
 
     initShell();
 }
@@ -264,6 +257,7 @@ void PyDUChainTest::testCrashes_data() {
     QTest::newRow("unicode escape char") << "print(\"\\xe9\")";
     QTest::newRow("augassign") << "a = 3\na += 5";
     QTest::newRow("delete") << "a = 3\ndel a";
+    QTest::newRow("double_comprehension") << "q = [[x for x in a] + [x for x in a] for y in b]";
     QTest::newRow("for_else") << "for i in range(3): pass\nelse: pass";
     QTest::newRow("for_while") << "while i < 4: pass\nelse: pass";
     QTest::newRow("ellipsis") << "a[...]";
@@ -277,6 +271,7 @@ void PyDUChainTest::testCrashes_data() {
     QTest::newRow("whatever") << "for attr in updated:\n    "
                                  "    getattr.update";
     QTest::newRow("return_outside_function") << "return 3";
+    QTest::newRow("return_context_outside_function") << "return [x for x in range(3)]";
     QTest::newRow("paren_attrib_access") << "a = (xxx or yyy).zzz";
     QTest::newRow("func_call") << "a = xxx.func(yyy.zzz)";
     QTest::newRow("comprehension_attrib") << "a = [foo for foo in bar].baz";
@@ -546,8 +541,6 @@ void PyDUChainTest::testCannotOverwriteBuiltins()
     Declaration* d = ds.first();
     QVERIFY(d);
     QVERIFY(d->abstractType());
-    QEXPECT_FAIL("can_have_custom", "not implemented", Continue);
-    QEXPECT_FAIL("can_have_custom3", "not implemented", Continue);
     QCOMPARE(d->abstractType()->toString(), expectedType);
 }
 
@@ -625,7 +618,7 @@ public:
     bool found;
     KTextEditor::Range searchingForRange;
     QString searchingForIdentifier;
-    virtual void visitAttribute(AttributeAst* node) {
+    void visitAttribute(AttributeAst* node) override {
         auto r = KTextEditor::Range(0, node->startCol, 0, node->endCol);
         qDebug() << "Found attr: " << r << node->attribute->value << ", looking for: " << searchingForRange << searchingForIdentifier;
         if ( r == searchingForRange && node->attribute->value == searchingForIdentifier ) {
@@ -634,7 +627,7 @@ public:
         }
         AstDefaultVisitor::visitAttribute(node);
     }
-    virtual void visitFunctionDefinition(FunctionDefinitionAst* node) {
+    void visitFunctionDefinition(FunctionDefinitionAst* node) override {
         auto r = KTextEditor::Range(0, node->name->startCol, 0, node->name->endCol);
         qDebug() << "Found func: " << r << node->name->value << ", looking for: " << searchingForRange << searchingForIdentifier;
         qDebug() << node->arguments->vararg << node->arguments->kwarg;
@@ -660,7 +653,7 @@ public:
         }
         AstDefaultVisitor::visitFunctionDefinition(node);
     }
-    virtual void visitClassDefinition(ClassDefinitionAst* node) {
+    void visitClassDefinition(ClassDefinitionAst* node) override {
         auto r = KTextEditor::Range(0, node->name->startCol, 0, node->name->endCol);
         qDebug() << "Found cls: " << r << node->name->value << ", looking for: " << searchingForRange << searchingForIdentifier;
         if ( r == searchingForRange && node->name->value == searchingForIdentifier ) {
@@ -669,7 +662,7 @@ public:
         }
         AstDefaultVisitor::visitClassDefinition(node);
     }
-    virtual void visitImport(ImportAst* node) {
+    void visitImport(ImportAst* node) override {
         foreach ( const AliasAst* name, node->names ) {
             if ( name->name ) {
                 qDebug() << "found import" << name->name->value << name->name->range();
@@ -763,7 +756,7 @@ public:
     QString searchingForType;
     TopDUContextPointer ctx;
     bool found;
-    virtual void visitName(NameAst* node) {
+    void visitName(NameAst* node) override {
         if ( node->identifier->value != "checkme" ) return;
         QList<Declaration*> decls = ctx->findDeclarations(QualifiedIdentifier(node->identifier->value));
         if ( ! decls.length() ) {
@@ -796,6 +789,7 @@ void PyDUChainTest::testTypes()
     visitor->searchingForType = expectedType;
     visitor->visitCode(m_ast.data());
     QEXPECT_FAIL("lambda", "not implemented: aliasing lambdas", Continue);
+    QEXPECT_FAIL("tuple_unsure", "not implemented, rework DeclarationBuilder::sourcesOfAssignment", Continue);
     QCOMPARE(visitor->found, true);
 }
 
@@ -818,7 +812,15 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("str") << "checkme = \"foo\"" << "str";
     QTest::newRow("bytes") << "checkme = b\"foo\"" << "bytes";
 
+    QTest::newRow("function_arg_scope") <<  "class Foo:\n"
+                                            "    a = 3\n"
+                                            "    def func(self, x=a):\n"
+                                            "        return x\n"
+                                            "f = Foo()\n"
+                                            "checkme = f.func()" << "int";
     QTest::newRow("with") << "with open('foo') as f: checkme = f.read()" << "str";
+    QTest::newRow("arg_after_vararg") << "def func(x, y, *, z:int): return z\ncheckme = func()" << "int";
+    QTest::newRow("arg_after_vararg_with_default") << "def func(x=5, y=3, *, z:int): return z\ncheckme = func()" << "int";
 
     QTest::newRow("list_access_right_open_slice") << "some_list = []; checkme = some_list[2:]" << "list";
     QTest::newRow("list_access_left_open_slice") << "some_list = []; checkme = some_list[:2]" << "list";
@@ -866,6 +868,31 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("tuple_simple2") << "mytuple = 3, 5.5\nfoobar, checkme = mytuple" << "float";
     QTest::newRow("tuple_simple3") << "mytuple = 3, 5.5, \"str\", 3, \"str\"\na, b, c, d, checkme = mytuple" << "str";
 
+    QTest::newRow("tuple_single") << "checkme = 4," << "tuple";
+    QTest::newRow("tuple_single2") << "checkme, = 4," << "int";
+    QTest::newRow("tuple_single3") << "mytuple = 4,\ncheckme, = mytuple" << "int";
+
+    QTest::newRow("tuple_ext_unpack") << "mytuple = 3, 5.5\nfoobar, *starred, checkme = mytuple" << "float";
+    QTest::newRow("tuple_ext_unpack2") << "mytuple = 3, 5.5\nfoobar, *checkme, another = mytuple" << "list";
+    QTest::newRow("tuple_ext_unpack3") << "mytuple = 3, 5.5\nfoobar, *checkme = mytuple" << "list of float";
+    QTest::newRow("tuple_ext_unpack4") << "mytuple = 3, 5.5\n*checkme, = mytuple" << "list of unsure (int, float)";
+
+    QTest::newRow("tuple_nested") << "mytuple = 3, ('foo', 5.5)\ncheckme, foobar = mytuple" << "int";
+    QTest::newRow("tuple_nested2") << "mytuple = 3, ('foo', 5.5)\nfoobar, (checkme, other) = mytuple" << "str";
+    QTest::newRow("tuple_nested3") << "mytuple = ((7, 'foo'), 5.5), 3\n((baz, checkme), other), foo = mytuple" << "str";
+
+    // This isn't actually defined behaviour, but it works in CPython, so people use it...
+    QTest::newRow("tuple_nested_ext") << "mytuple = (2, ('foo', 'bar', 6), 7)\na, (b, *checkme, c), *d = mytuple" << "list of str";
+
+    QTest::newRow("tuple_multi_assign") << "mytuple = 2, 'foo'\ncheckme = a = mytuple" << "tuple";
+    QTest::newRow("tuple_multi_assign2") << "mytuple = 2, 'foo'\ncheckme, a = b = mytuple" << "int";
+
+    QTest::newRow("list_unpack") << "mylist = [1, 2, 3]\ncheckme, b, c = mylist" << "int";
+    QTest::newRow("list_unpack2") << "mylist = [1, 'x', 3]\ncheckme, b, c = mylist" << "unsure (int, str)";
+
+    QTest::newRow("list_ext_unpack") << "mylist = [1, 2, 3]\n*checkme, foo = mylist" << "list of int";
+    QTest::newRow("list_ext_unpack2") << "mylist = [1, 'x', 3]\n*checkme, foo = mylist" << "list of unsure (int, str)";
+
     QTest::newRow("if_expr_sure") << "checkme = 3 if 7 > 9 else 5" << "int";
 
     QTest::newRow("unary_op") << "checkme = -42" << "int";
@@ -900,6 +927,8 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("args_type") << "def myfun(*args): return args[0]\ncheckme = myfun(3)" << "int";
     QTest::newRow("kwarg_type") << "def myfun(**args): return args[0]\ncheckme = myfun(a=3)" << "int";
 
+    QTest::newRow("tuple_unsure") << "q = (3, str())\nq=(str(), 3)\ncheckme, _ = q" << "unsure(int, str)";
+
     QTest::newRow("call_class") << "class Foo:\n"
                                     "    def __call__(self):\n"
                                     "         return 0\n"
@@ -1391,9 +1420,9 @@ void PyDUChainTest::testVariableCreation_data()
     QTest::newRow("tuple_wrong") << "a, b = 3" << QStringList{"a", "b"} << QStringList{"mixed", "mixed"};
     QTest::newRow("tuple_unpack_inplace") << "a, b = 3, 5.5" << QStringList{"a", "b"} << QStringList{"int", "float"};
     QTest::newRow("tuple_unpack_indirect") << "c = 3, 3.5\na, b = c" << QStringList{"a", "b"} << QStringList{"int", "float"};
-    QTest::newRow("tuple_unpack_stacked_inplace") << "a, (b, c) = 1, 2, 3.5" << QStringList{"a", "b", "c"}
+    QTest::newRow("tuple_unpack_stacked_inplace") << "a, (b, c) = 1, (2, 3.5)" << QStringList{"a", "b", "c"}
                                                                              << QStringList{"int", "int", "float"};
-    QTest::newRow("tuple_unpack_stacked_indirect") << "d = 3.5, 3, 1\na, (b, c) = d"
+    QTest::newRow("tuple_unpack_stacked_indirect") << "d = 3.5, (3, 1)\na, (b, c) = d"
                                                    << QStringList{"a", "b", "c"} << QStringList{"float", "int", "int"};
     QTest::newRow("unpack_from_list_inplace") << "a, b = [1, 2, 3]" << QStringList{"a", "b"} << QStringList{"int", "int"};
     QTest::newRow("unpack_from_list_indirect") << "c = [1, 2, 3]\na, b = c" << QStringList{"a", "b"}
@@ -1405,3 +1434,44 @@ void PyDUChainTest::testVariableCreation_data()
                                                                                 << QStringList{"int", "int", "float"};
 }
 
+void PyDUChainTest::testCleanupMultiplePasses()
+{
+    for ( int j = 0; j < 20; j++ ) {
+        ReferencedTopDUContext top = parse("from testCleanupMultiplePasses import foo\ndef fonc(): return 3+2j\nfoo.foo.func = fonc");
+    }
+}
+
+void PyDUChainTest::testManyDeclarations()
+{
+    ReferencedTopDUContext top = parse("from testManyDeclarations import test\nk=test.Foo()");
+}
+
+void PyDUChainTest::testComments()
+{
+    QFETCH(QString, code);
+
+    auto top = parse(code);
+    QVERIFY(top);
+    DUChainReadLocker lock;
+
+    auto decls = top->findDeclarations(QualifiedIdentifier("a"));
+    QCOMPARE(decls.size(), 1);
+    auto a = decls.first();
+    QCOMPARE(a->comment(), QByteArray("comment"));
+
+    decls = top->findDeclarations(QualifiedIdentifier("b"));
+    if ( decls.isEmpty() ) {
+        decls = top->childContexts().last()->findDeclarations(QualifiedIdentifier("b"));
+    }
+    auto b = decls.first();
+    QCOMPARE(b->comment(), QByteArray());
+}
+
+void PyDUChainTest::testComments_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("variable") << "b=5\n\"\"\"comment\"\"\"\na=5\nb=5";
+    QTest::newRow("function") << "def a():\n    \"\"\"comment\"\"\"\n    b=5";
+    QTest::newRow("class") << "class a:\n    \"\"\"comment\"\"\"\n    b=5";
+}
