@@ -26,6 +26,7 @@
 #define PYDUCHAINTEST_H
 
 #include <QObject>
+#include <QTemporaryDir>
 #include "ast.h"
 #include <serialization/indexedstring.h>
 #include <language/duchain/topducontext.h>
@@ -44,7 +45,7 @@ class PyDUChainTest : public QObject
     public:
         explicit PyDUChainTest(QObject* parent = 0);
         void initShell();
-        virtual ~PyDUChainTest();
+        ~PyDUChainTest() override;
 
         KDevelop::ReferencedTopDUContext parse(const QString& code);
         
@@ -96,10 +97,15 @@ class PyDUChainTest : public QObject
         void testProblemCount_data();
         void testHintedTypes();
         void testHintedTypes_data();
+        void testCleanupMultiplePasses();
+        void testComments();
+        void testComments_data();
+        void testManyDeclarations();
 
 
     private:
         QList<KDevelop::TestFile*> createdFiles;
+        QTemporaryDir testDirOwner;
         QDir testDir;
         QDir assetsDir;
         QString lastTest;
