@@ -62,9 +62,9 @@ void PEP8KCModule::apply()
 }
 
 static QString defaultPep8Path() {
-    QString result = QStandardPaths::findExecutable("pep8-python3");
+    QString result = QStandardPaths::findExecutable("pycodestyle3");
     if (result.isEmpty()) {
-        result = QStandardPaths::findExecutable("pep8");
+        result = QStandardPaths::findExecutable("pycodestyle");
     }
     return result;
 }
@@ -111,7 +111,7 @@ QIcon PEP8KCModule::icon() const
 
 QString PEP8KCModule::name() const
 {
-    return i18n("Python style checking");
+    return i18n("Python Style Checking");
 }
 
 PEP8KCModule::~PEP8KCModule()
